/**
 * 广点通移动广告SDK - 渲染器
 * 支持Banner广告、应用墙
 * @version 1.2
 * @author damonpeng@tencent.com
 */
(function(GDTMOB){
	var MB = {};

	GDTMOB.version	= 1.1;		// 当前js版本号
	
	GDTMOB.posid	= null;		// 广告位ID
	GDTMOB.orderid	= null;		// 当前广告id
	GDTMOB.wrap		= null;
	GDTMOB.adData	= null;		// 当前广告的数据
	GDTMOB.playcfg	= null;		// 展示配置
	GDTMOB.ACTTYPE_DOWNLOAD = 1;// 下载行为的广告

	GDTMOB.closeBtn	= null;
	GDTMOB.downloadBtn	= null;		// 下载按钮
	MB._loadAding	= false;	// 正在获取广告

	/**
	 * 获取参数
	 * @param  {string} name	参数名
	 * @return {string} 参数值
	 */
	MB._parameters = null;
	MB._getParameter = function(name) {
		if (! MB._parameters) {
			var search = location.search.substring(1),
				hash = location.hash.substring(1),
				ps = {}, tmp;

			tmp = ([hash, search].join('&')).split('&');
			for(var i=0, l=tmp.length; i<l; i++) {
				var t = tmp[i].split('=');
				ps[t[0]] = t[1] || '';
			}

			MB._parameters = ps;
		}

		return MB._parameters[name] || '';
	};

	/**
	 * 简单加载JSONP数据， 
	 * @todo  需要监控超时以及无法解析的情况
	 * @param  {[type]} url    [description]
	 * @param  {[type]} onload [description]
	 * @param  {[type]} opts   [description]
	 * @return {[type]}        [description]
	 */
	MB._loadData = function(url, onload, opts) {
		var sender;

		// @todo 替换成走SDK实现

		opts = opts || {};

		sender = document.createElement('script');
		sender.charset = opts.charset || 'gb2312';
		sender.onload = (function(onload){
			return function(){
				onload();
				this.onload = null;
				this.parentNode.removeChild(this);
			};
		})(onload || function(){});

		sender.src = url;

		document.body.appendChild(sender);
	};

	/**
	 * 创建JS与Client的gdtmsg通信器
	 * @return {object} 
	 */
	MB._sender = null;
	MB._getMessageSender = function() {
		var ifr;

		// 不存在则创建
		//if (! (ifr=MB._sender)) {
			ifr = document.createElement('iframe');
			MB._sender = ifr;
			ifr.id = 'gdtmob-notify-' + (new Date).getTime();
			ifr.style.display = 'none';
		//}

		return ifr;
	};
	
	/**
	 * 生成请求串
	 * @param  {object} params	参数
	 * @return {string}     	请求串
	 */
	MB._genHTTPParams = function(params) {
		var req = [];

		for(var p in params) {
			req.push(p +'='+ encodeURIComponent(params[p]));
		}

		return req.join('&');
	};

	/**
	 * JS与Client的通信
	 * @param  {string} action 行为
	 * @param  {string} msg 需要传递的信息串
	 * @return {undefiend} 
	 */
	GDTMOB.sendMessage = function(action, params) {
		var requestUrl, ps,
			sender = MB._getMessageSender();

		ps = typeof params==='object' ? MB._genHTTPParams(params) : '';

		requestUrl = 'gdtmsg://e.qq.com/'+ action + (ps ? '?' : '') + ps;
		sender.src = requestUrl;

		// @todo，队列加载，保证请求有序，不保证回调有序。

		sender.parentNode || document.body.appendChild(sender);
	};
	GDTMOB.receiverMsg = function(url){
		window.title="hell";
	}

	GDTMOB.getDisplayPixel = function() {
		var dd = MB._getParameter('dd'), dpi;

		if (! dd) {
			dpi = window.devicePixelRatio;
		} else {
			dpi = dd / 160;
		}

		// 目前支持240.320.480.640
		return Math.round(320 * dpi);
	};

	// 格式化运营商参数
	MB._formatCarrier = function(op) {
		var unknown = 0,
			carrier, rpt,
			mapping = {
				'46000'	: 1,	// 中国移动
				'46001'	: 2,	// 中国联通
				'46002'	: 1,	// 中国移动
				'46003'	: 3,	// 中国电信
				'46006'	: 2,	// 中国联通
				'46007'	: 1,	// 中国移动
				'46020'	: 1,	// 中国铁通，并入中国移动
				'46005'	: 3,	// 中国电信
			};

		carrier = mapping[op] || unknown;

		// 统计下
		rpt = {
			'1': 'cm',
			'2': 'cuc',
			'3': 'ctc'
		};
		rpt[unknown] = 'ukn';
		GDTMOB.hotSend('carrier.'+ rpt[carrier+'']);
		
		return carrier;
	};

	// 格式化联网方式参数
	MB._formatConn = function(pn, dn) {
		var unknown = 0, rpt,
			pnMapping = {
				// 1:wifi, 2:2G, 3:3G, 4:4G
				// @todo 这个表的划分待确定
				'1'		: 2,  // GPRS
				'2'		: 2,  // EDGE
				'3'		: 3,  // UMTS
				'4'		: 3,  // CDMA
				'5'		: 3,  // EVDO revision 0
				'6'		: 3,  // EVDO revision A
				'7'		: 3,  // 1xRTT
				'8'		: 4,  // HSDPA
				'9'		: 4,  // HSUPA
				'10'	: 3,  // HSPA
				'11'	: 3,  // iDen
				'12'	: 3,  // EVDO revision B
				'13'	: 4,  // LTE
				'14'	: 3,  // eHRPD
				'15'	: 4,  // HSPA+
			},
			conn;

		// 2G算作未知，暂无对2G的定向
		conn = dn=='wi' ? 1 : (pn==1||pn==2 ? unknown : (pnMapping[pn] || unknown));
		
		// 统计下
		rpt = {
			'1': 'wi',
			'2': '2g',
			'3': '3g',
			'4': '4g'
		};
		rpt[unknown] = 'ukn';
		GDTMOB.hotSend('conn.'+ rpt[conn+'']);

		return conn;
	};

	/**
	 * 获取拉取广告的cgi地址
	 * @return {[type]} [description]
	 */
	GDTMOB.getBaseCgi = function() {
		var cgi = 'http://mi.gdt.qq.com/gdt_mview.fcg?',
			isTest = MB._getParameter('testad') || false;

		// 是否测试广告
		isTest && (cgi += '&reqtype=1&');

		return cgi;
	};

	/**
	 * 获取广告数据，轮播前，需要从SDK获取最新的参数（位置信息、横竖屏等）
	 * @return {undefined} 
	 */
	GDTMOB.loadAd = function(params) {
		var posid = MB._getParameter('posid'), url,
			sender, rf, ps,
			adtype = MB._getParameter('adtype'), callback, cgi;

		MB._REQUEST_TIME = +new Date;

		// 先判断下时间间隔是否合理，防止错误引用导致ddos效果。放开2s，避免执行慢
		if (MB._loadAding || (MB._RESPONSE_TIME && MB._rf>0 && (MB._REQUEST_TIME - MB._RESPONSE_TIME < (MB._rf-2000)))) {
			return false;
		}
		MB._loadAding = true;

		// 更新参数
		if (typeof params === 'object') {
			MB._parameters = MB._parameters || {};
			for(var x in params) {
				MB._parameters[x] = params[x];
			}
		}

		if (adtype === 'maw') {
			callback = 'GAppWall.render';
			cgi = GAppWall.cgi;

			GAppWall.init();
		} else {
			callback = 'GBanner.render';
			cgi = GBanner.cgi;

			GAppWall.init();
		}

		ps = {
			posid	: posid,
			count	: 1,
			posw	: MB._getParameter('adw'),
			posh	: MB._getParameter('adh'),
			ext		: JSON.stringify({
						req: {
							c_pkgname	: MB._getParameter('an'),  // 当前app包名
							muid		: MB._getParameter('did'),
							conn		: MB._formatConn(MB._getParameter('pn'), MB._getParameter('dn')),
							carrier		: MB._formatCarrier(MB._getParameter('op')),
							lat			: Math.round((MB._getParameter('lat')||0)*1e6),
							lng			: Math.round((MB._getParameter('lng')||0)*1e6)
						}
					}),
			callback: callback
		};

		url = cgi +'&'+ MB._genHTTPParams(ps) +'&r='+ Math.random();

		MB._loadData(url, function(){
			MB._loadAding = false;
			MB._RESPONSE_TIME = +new Date;
		});

		GDTMOB.posid = posid;
		GDTMOB.wrap = document.getElementById('wrap');
		// 如果有关闭按钮
		if(GDTMOB.closeBtn = document.getElementById('closeBtn')) {
			GDTMOB.closeBtn.onclick = GDTMOB.close;
		}
	};

	/**
	 * 获取广告数据，轮播前，需要从SDK获取最新的参数（位置信息、横竖屏等）
	 * @return {undefined} 
	 */
	GDTMOB.reLoadAd = function() {
		GDTMOB.sendMessage('fillparams', {'callback': 'GDTMOB.loadAd'});
	};

	/**
	 * cgi的回调函数，通用处理部分
	 * @param  {object} d cgi返回的广告数据
	 * @return {undefined}
	 */
	GDTMOB.render = function(d, callback) {
		var params, res, data, tmpdata;

		if(d && d.ret===0 && (res=d.data[GDTMOB.posid]) && (res.ret===0) && (data=res.list)) {
			// 就算是重复的广告，也要再次渲染下，避免点击链接串失效
			if(res.cfg) {
				GDTMOB.playcfg = res.cfg.playcfg || {};
			}

			// 按广告id存储
			tmpdata = {};
			for (var i=0, l=data.length; i<l; i++) {
				tmpdata[data[i].cl] = data[i];
			};
			GDTMOB.adData = tmpdata;
			tmpdata = null;

			params = {
				apps		: MB._getAdPackageName(data),  // @todo 这里获取多组，逗号分隔
				// version		: '',	// app版本号，目前暂无
				infotype	: 1,	// 1：是否安装，2：是否已下载，3：app版本信息。
				callback	: callback
			};
// @todo 临时去掉
			GDTMOB.sendMessage('appsinfo', params);
			// GBanner.showAd();  // @todo mock

			// @todo 不能同时请求sdk，否则一条请求不会及时返回

			// SDK会通过调用GDTMOB.timepot 传入之前记录的时间点
			!MB._isSend && setTimeout(function() {
				GDTMOB.sendMessage('marktime', {callback: 'GDTMOB.timepot'});
			}, 1000);
		}

		// 上报返回码
		setTimeout((function(rpt){
			return function(){
				MB._retSend(rpt, 'i.gdt.qq.com', '/gdt_mview.fcg?'+GDTMOB.posid, MB._RESPONSE_TIME-MB._REQUEST_TIME);
			};
		})(d.rpt), 500);

		return d;
	};

	/**
	 * 真正展示广告，这里由SDK回调
	 */
	GDTMOB.showAd = function(appsinfo) {
		var tplTxt, tplImg, rf,
			preEl, nextEl,
			cfg = GDTMOB.playcfg,
			data = GDTMOB.adData,
			posid = GDTMOB.posid,
			w='100%', h='100%', wrapEl;

		// @todo 过滤掉已经安装的

		// 可能由于拉取的app已经在手机上安装过了
		

		return data;
	};


	/**
	 * 关闭当前界面的广告展示
	 * @return {[type]} [description]
	 */
	GDTMOB.close = function() {
		// 统计下点击关闭的人有多少
		var purl = GDTMOB.hotSend('close', true);
		
		// 同时发ping上报本次关闭行为，后续切换为cgi接收
		GDTMOB.sendMessage('destroy', {url:purl});

		return false;
	};

	/**
	 * 曝光上报
	 * @return {[type]} [description]
	 */
	GDTMOB.exposure = function(apurls) {
		var pingurl, viewids, count;

		pingurl = 'http://v.gdt.qq.com/gdt_stats.fcg?';

		typeof apurls!=='object' && (apurls = [apurls]);
		viewids = [];
		count=apurls.length;
		for (var i=0; i<count; i++) {
			var mat;
			(mat = apurls[i].match(/viewid=([^&]*)/))
				&& mat[1]
				&& viewids.push('viewid'+ i +'='+ mat[1]);
		};

		if (viewids.length < 1) {
			return;
		}

		pingurl += 'count='+ count +'&'+ viewids.join('&');
		
		// @todo 去掉延时
		setTimeout((function(url){
			return function(){
				GDTMOB.sendMessage('ping', {
					url		: url,
					type	: 1,	// 可选，默认为0，0：普通上报，1：曝光上报
					method	: count>4 ? 'post' : 'get'
					// data : {}
				});
			};
		})(pingurl), 100);
	};

	/**
	 * 是否下载
	 */
	MB._isDownload = function() {
		// @todo 如何找到具体的某个
		return GDTMOB.adData['ext'] && GDTMOB.adData['ext']['alist'] && 
			GDTMOB.adData['ext']['alist']['2022'] ;
	};

	/**
	 * 从广告数据中获取PackageName
	 * @param  {[type]} data [description]
	 * @return {[type]}      [description]
	 */
	MB._getAdPackageName = function(data) {
		var pkgname = [];

		!data && (data = GDTMOB.adData);

		for (var x in data) {
			extdata = data[x].ext || {};
			extdata.alist && extdata.alist['2022'] && (pkgname.push(extdata.alist['2022'].aid||''));
		}

		return pkgname.join(',');
	};

	/**
	 * 点击广告链接
	 * @param  {[type]} el [description]
	 * @return {[type]}    [description]
	 */
	GDTMOB.click = function(el, type, via) {
		var link = el.href || '',
			callback;

		GDTMOB.orderid = el.getAttribute('data-orderid') || '';
		GDTMOB.downloadBtn = null;
	
		// 打开链接 or 下载？
		if (type === 'open') {
			GDTMOB.sendMessage('website', {url: link});
		} else if (type === 'download') {
			// 先请求下点击链接，返回真实内容
			link += '&callback=&r='+Math.random();
			GDTMOB.downloadBtn = el;
			MB._loadData(link);

			callback = via==='appwall' ? 'GAppWall.download' : 'GDTMOB.download';
		} else if (type === 'popup') {
			var appid = el.getAttribute('data-appid');

			// ico 是否返回图标url（可选，1 需要 0 不需要）
			MB._loadData('http://d.gdt.qq.com/fcg-bin/gdt_appdetail.fcg?appid='+ appid +'&ico=1&cb=GAppWall.popupDetail', null, {charset:'utf-8'});
		};

		// 防止继续冒泡，导致mask被关
		event.stopPropagation();

		return false;
	};

	/**
	 * 下载
	 */
	GDTMOB.download = function(d) {
		var data = GDTMOB.adData[GDTMOB.orderid],
			commdata = {};

		if (d) {
			// 正常，返回下载链接
			if(d.ret === 0) {
				commdata.clickid = d.data.clickid;
				commdata.pkgname = MB._getAdPackageName(data);  // @todo 获取对应的

				commdata.dlping = 'http://c.gdt.qq.com/gdt_trace_a.fcg?actionid=5&targettype=6&targetid='+ data.targetid +'&sellerid='+ data.targettype +'&clickid='+ clickid;
				commdata.dledping = 'http://c.gdt.qq.com/gdt_trace_a.fcg?actionid=7&targettype=6&targetid='+ data.targetid +'&sellerid='+ data.targettype +'&clickid='+ clickid;

				commdata.dstlink = d.data.dstlink;
				// @todo 系统版本、机型、网络连接方式、运营商
				commdata.dlerrping = GDTMOB.hotSend('download.'+ GDTMOB.posid +'.err.'+ (commdata.pkgname ? commdata.pkgname.replace(/\./g, '_') : 'null'), true); // 报点击流
			} 
		}

		// 返回码上报
		setTimeout(function(){
			MB._retSend(d.ret, 'c.gdt.qq.com', '/gdt_mclick.fcg?'+GDTMOB.posid);
		}, 500);

		return commdata;
	};

	/**
	 * 整体测速上报
	 * 
		1. 进入SDK（sdk可控）
		2. 创建webview，注入html（sdk可控）
		3. 拉取广告模板页（此时间点由js补充）
		4. 拉取广告数据（此时间点由js补充）
		5. 广告展示完毕（此时间点由js补充）
	 */
	MB._isSend = false;
	GDTMOB.timepot = function(stamps) {
		var url, mark, i, l, idx, timing, sdkt0, webt0, dots = [], dt;
		// 只报一次
		if (MB._isSend) {
			return;
		}
		MB._isSend = true;

		// 检查是否数组
		if (Object.prototype.toString.call(stamps).slice(8, -1).toLowerCase() == 'array') {
			stamps.push(window._PAGE_TIME);
			stamps.push(MB._RESPONSE_TIME);

			// 不抽样
			url = 'http://isdspeed.qq.com/cgi-bin/r.cgi?flag1=175&flag2=379&flag3=1&flag5=1';

			mark = ['navigationStart','unloadEventStart','unloadEventEnd','redirectStart','redirectEnd','fetchStart','domainLookupStart','domainLookupEnd','connectStart','connectEnd','requestStart','responseStart','responseEnd','domLoading','domInteractive','domContentLoadedEventStart','domContentLoadedEventEnd','domComplete','loadEventStart','loadEventEnd'];

			perf = window.performance || window.webkitPerformance;
			if (perf && (timing = perf.timing)) {
				webt0 = timing[mark[0]];
				for (i=1, l=mark.length; i<l; i++) {
					dt = timing[mark[i]] - webt0;
					dots.push(i +'='+ (dt>0?dt:0));
				}
			}

			sdkt0 = stamps[0];
			for (i=1, idx=20, l=stamps.length; i<l; i++, idx++) {
				dots.push(idx +'='+ (stamps[i]-sdkt0));
			};

			GDTMOB.sendMessage('ping', {url: url +'&'+ dots.join('&') +'&r='+ Math.random()});
		}
	};

	/**
	 * 返回码上报
	 * @param  {[type]} ret [description]
	 * @return {[type]}     [description]
	 */
	MB._retSend = function(ret, domain, cgi, duration) {
		var url, type, time;

		// 抽样
		if (Math.random() > 0.01) {
			return;
		};

		typeof ret==='undefied' && (ret = 51);
		type = ret>=50 ? 2 : 1;
		url = 'http://c.isdspeed.qq.com/code.cgi?domain='+ domain +'&cgi='+ cgi +'&code='+ ret +'&rate=100';  // 抽样1/100

		url += '&type='+ type +'&time='+ (duration||0);

		GDTMOB.sendMessage('ping', {url: url +'&r='+ Math.random()});
	};

	/**
	 * 热点统计
	 * @param  {string} type    上报值
	 * @param  {boolean} justGen 是否仅生成上报url（不会ping），默认会发送此ping
	 * @return {boolean|undefied}
	 */
	GDTMOB.hotSend = function(type, justGen) {
		// 统计下点击关闭的人有多少
		var purl = [
			'http://pinghot.qq.com/pingd',
			'?dm=gdt.qq.com.hot',
			'&url=', escape(location.pathname),
			'&tt=-',
			'&hottag=gdtmob.display.'+ type,
			'&hotx=9999',
			'&hoty=9999',
			'&rand=', Math.random()
		].join('');

		if (justGen) {
			return purl;
		} else {
			GDTMOB.sendMessage('ping', {url: purl});
		}
	};

	// 自建调试台
	MB.console = function() {
		var con = document.getElementById('console'),
			args;

		args = Array.prototype.slice.call(arguments);
		for(var i=0, l=args.length; i<l; i++) {
			if (Object.prototype.toString.call(args[i]).slice(8, -1).toLowerCase() == 'object') {
				var tmp=[];
				for(var x in args[i]) {
					tmp.push(x +':'+ args[i][x]);
				}
				args[i] = tmp.join('|');
			}
		}

		if (! con) {
			con = document.createElement('div');
			con.id = 'console';
			con.style.cssText = 'position:absolute;bottom:0;height:120px;width:100%;background:#f60;color:#fff;font-size:24px;overflow:auto';
			document.body.appendChild(con);
		}

		GDTMOB.sendMessage('show');

		con.innerHTML += '<br>'+ args.join(', ');
	};

	/**
	 * 模板填充 QBL.tmpl
	 */
	(function(qt){
		var cache = {};
		qt.tmpl = function tmpl(str, data){
			// Figure out if we're getting a template, or if we need to
			// load the template - and be sure to cache the result.
			var fn = !/\W/.test(str) ?
			cache[str] = cache[str] ||
			tmpl(document.getElementById(str).innerHTML) :
			// Generate a reusable function that will serve as a template
			// generator (and which will be cached).
			new Function("obj",
				"var p=[],print=function(){p.push.apply(p,arguments);};" +
				
				// Introduce the data as local variables using with(){}
				"with(obj){p.push('" +
				
				// Convert the template into pure JavaScript
				str
					.replace(/[\r\t\n]/g, " ")
					.split("<%").join("\t")
					.replace(/((^|%>)[^\t]*)'/g, "$1\r")
					.replace(/\t=(.*?)%>/g, "',$1,'")
					.split("\t").join("');")
					.split("%>").join("p.push('")
					.split("\r").join("\\'")
				+ "');}return p.join('');");
			
			// Provide some basic currying to the user
			return data ? fn( data ) : fn;
		};
	})(GDTMOB);
})(window.GDTMOB = window.GDTMOB || {});

/**
 * Banner 广告
 */
(function(GBanner){
	var GB = {};

	GBanner.cgi = GDTMOB.getBaseCgi() +'adposcount=1';  // 广告数据cgi
	GB._playTimer	= null;		// 轮播的定时器
	GB._rf			= null;		// 轮播时延
	GB._count		= 0;		// 拉取广告的次数

	/**
	 * 初始化入口，默认参数获取等
	 */
	GBanner.init = function() {
		var rf;

		window.setTimeout(function(){
				GDTMOB.sendMessage('log', {info: "init"});
		},2000);

		// 请求频次
		rf = parseInt(MB._getParameter('rf'), 10);
		if (adtype==='mb' && rf > 0) {
			// admob(30 ~ 120 seconds)，默认不轮播。若要延长上限，不能大于点击链接串的有效时间
			(rf<30000) && (rf = 30000);
			(rf>120000) && (rf = 120000);
			rf = 5000;
		} else {
			rf = -1;  // 不轮播
		}

		//GB._rf = rf;
		GB._rf = 5000;
	};

	/**
	 * cgi的回调函数，填充模板的
	 * @param  {object} d cgi返回的广告数据
	 * @return {undefined}
	 */
	GBanner.render = function(d) {
		var params;
		GB._count ++;

		d = GDTMOB.render(d, 'GBanner.showAd');
	};

	/**
	 * 真正展示广告，这里由SDK回调
	 */
	GBanner.showAd = function(appsinfo) {
		var tplTxt, tplImg, rf,
			preEl, nextEl,
			cfg = GDTMOB.playcfg,
			data,
			posid = GDTMOB.posid,
			w='100%', h='100%', wrapEl;

		// 调下通用的，会对数据做过滤处理
		data = GDTMOB.showAd(appsinfo) || {};
		// banner只有一个广告，就取第一个
		for (var orderid in data) {
			data = data[orderid];
			break;
		};

		// 是否全部被过滤了
		if (! data) {
			// 如果已经有广告展示了，则不更换；只要第一次失败hide下就行了
			if (GB._count <= 1) {
				GDTMOB.sendMessage('hide');
			}
			return;
		}

		// 就算是重复的广告，也要再次渲染下，避免点击链接串失效
		if(cfg) {
			cfg.w && (w=cfg.w);
			cfg.h && (h=cfg.h);
		}

		wrapEl = GDTMOB.wrap;
		if(wrapEl.parentNode.className.indexOf('size') < 0) {
			wrapEl.parentNode.className += ' size'+ GDTMOB.getDisplayPixel();
		}

		preEl = wrapEl.firstChild;
		nextEl = document.createElement('a')
		nextEl.target = '_blank';
		nextEl.href = data.rl;
		nextEl.onclick = function(event){
			GB._click(this);
			event.preventDefault();
			return false;
		};
		// 可能图片加载慢，等加载完后才show  @todo 对已经cache的？ oncompelete
		nextEl.innerHTML = '<img src="'+ data.img +'" width="'+ w +'" height="'+ h +'" onload="GBanner.showAdview" />';

		// 先追加元素，再缩减原来图片的高度
		wrapEl.appendChild(nextEl);

		setTimeout(function(){
			if (preEl) {
				preEl.style.cssText = 'font-size:0;line-height:0';
				preEl.firstChild.style.cssText = 'height:0;-webkit-transition:height .5s linear;transition:height .5s linear';
				setTimeout(function(){
					preEl.parentNode.removeChild(preEl);
				}, 500);  // 晚点再移除，有的机器跑得慢
			}
		}, 0);

		GBanner.playRepeat();

		// 曝光上报
		GDTMOB.exposure(data.apurl);
	};

	/**
	 * 加载完图片后，才真正展示，避免白屏
	 * @return {[type]} [description]
	 */
	GBanner.showAdview = function() {
		GDTMOB.sendMessage('show');
	};

	/**
	 * 点击广告
	 * @param  {[type]} obj [description]
	 * @return {[type]}     [description]
	 */
	GB._click = function(el) {
		var mask = document.createElement('div'),
			btns = document.createElement('div'),
			link = el.href || '',
			tpl, isDownload = MB._isDownload();

		// banner广告去掉二次确认，二次确认里没有实际信息。 @todo 实验下线

		// 测试是否需要二次确认的点击率
		// 5%的流量需要二次确认; APP下载的一定要二次确认
		if (isDownload || GDTMOB.playcfg && GDTMOB.playcfg['1034']) {
			clearTimeout(MB._playTimer);  // 暂停轮播

			GDTMOB.closeBtn.style.display = 'none';
			
			// @todo 做成tmpl
			tpl = '<a href="'+ link +'" onclick="return GDTMOB.click(this,\'open\')" class="detail"><span>查看详情</span></a>';
			if (isDownload) {
				tpl += '<a href="'+ link +'&acttype='+ GDTMOB.ACTTYPE_DOWNLOAD +'" onclick="return GDTMOB.click(this,\'download\')" class="download"><span>直接下载</span></a>';
			}
			mask.className = 'mask';
			mask.innerHTML = tpl;

			// 点击消失
			mask.onclick = function(){
				GDTMOB.wrap.removeChild(this);
				GDTMOB.closeBtn.style.display = '';
				GBanner.playRepeat();  // 继续轮播
				return false;
			};
			GDTMOB.wrap.appendChild(mask);

			// 轮播频率后（默认30s）后，还是继续轮播
			MB._rf>0 && setTimeout(GBanner.playRepeat, MB._rf);

			GDTMOB.hotSend('firstclick');
		} else {
			GDTMOB.click(el, 'open');
			setTimtout(function(){  // @todo 没有队列，需要延时下
				GDTMOB.hotSend('webclick');
			}, 500);
		}
		
		return false;
	};

	/**
	 * 轮播
	 * @type {[type]}
	 */
	GBanner.playRepeat = function() {
		var rf = GB._rf;

		// 最大轮播上限50次
		if (rf>0 && GB._count<50) {
			clearTimeout(GB._playTimer);
			GB._playTimer = setTimeout(GDTMOB.reLoadAd, rf);
		}
	};

	/**
	 * 下载
	 * @return {[type]} [description]
	 */
	GBanner.download = function(d) {
		var commdata;

		commdata = GDTMOB.download(d);

		if (d) {
			// 正常，返回下载链接
			if(d.ret === 0) {
				GDTMOB.sendMessage('download', {
					url			: commdata.dstlink,
					pkgname		: commdata.pkgname,
					dlping		: commdata.dlping,	// 下载上报url
					dledping	: commdata.dledping,	// 下载完成上报url
					dlerrping	: commdata.dlerrping
				});

				// 提示用户开始下载了
				GDTMOB.downloadBtn && (GDTMOB.downloadBtn.innerHTML = '<span>下载中...</span>');
			} else if (d.data && commdata.dstlink) {
				// 如果遇到后台逻辑错误，也有可能返回正确的地址，但地址是详情页。此时调外部浏览器打开
				GDTMOB.sendMessage('website', {url: commdata.dstlink});
			} else {
				GDTMOB.downloadBtn && (GDTMOB.downloadBtn.innerHTML = '<span>重新下载</span>');
			}
		} else {
			GDTMOB.downloadBtn && (GDTMOB.downloadBtn.innerHTML = '<span>重新下载</span>');
		}
	};

})(window.GBanner = window.GBanner || {});

/**
 * 应用墙
 */
(function(){
	var GAW = {};

	GAppWall.cgi = GDTMOB.getBaseCgi() +'adposcount=10';  // 广告数据cgi
	
	/**
	 * 初始化入口，默认参数获取等
	 */
	GAppWall.init = function() {};

	/**
	 * cgi的回调函数，填充模板的
	 * @param  {object} d cgi返回的广告数据
	 * @return {undefined}
	 */
	GAppWall.render = function(d) {
		var params;

		d = GDTMOB.render(d, 'GAppWall.showAd');


		// @todo 展示列表
	};

	/**
	 * 真正展示广告，这里由SDK回调
	 */
	GAppWall.showAd = function(appsinfo) {
		var tpl, 
			cfg = GDTMOB.playcfg,
			data,
			posid = GDTMOB.posid,
			wrapDf, apurls,
			w='100%', h='100%', wrapEl;

		// 调下通用的，会对数据做过滤处理
		data = GDTMOB.showAd(appsinfo);

		// 是否全部被过滤了
		if (! data) {
			return;
		}

		// 就算是重复的广告，也要再次渲染下，避免点击链接串失效
		if(cfg) {
			cfg.w && (w=cfg.w);
			cfg.h && (h=cfg.h);
		}

		wrapEl = GDTMOB.wrap;
		/*
		if(wrapEl.parentNode.className.indexOf('size') < 0) {
			wrapEl.parentNode.className += ' size'+ GDTMOB.getDisplayPixel();
		}
		*/
	
		tpl = "\n"+ document.getElementById('tplItem').innerHTML;

		apurls = [];
		wrapDf = document.createDocumentFragment();
		for(var i in data) {
			var item = data[i],
				node = document.createElement('li');

			item.clickfn = 'GDTMOB.click(this,\'popup\');return false;';
			item.downloadfn = 'GDTMOB.click(this,\'download\');return false;';
			item.appid = item.uin || '';  // 残毒，投放端的linktarget最终输出为uin（当时仅考虑了商户QQ的情况）
			item.rl += '&acttype='+ GDTMOB.ACTTYPE_DOWNLOAD;
			
			node.innerHTML = GDTMOB.tmpl(tpl, item);
			wrapDf.appendChild(node);
			apurls.push(item.apurl);
		}

		// 隐藏loading
		document.getElementById('pageloading').style.display = 'none';

		// 展示广告
		wrapEl.appendChild(wrapDf);

		// 曝光上报
		GDTMOB.exposure(apurls);

		// 上拉刷新
		GAW.bindRefresh();
	};

	/**
	 * 下拉刷新
	 * @return {[type]} [description]
	 */
	GAW.isBindRefresh = false;
	GAW._touchData = {};
	GAW.bindRefresh = function() {
		if (GAW.isBindRefresh) {
			return;
		}
		GAW.isBindRefresh = true;
		document.addEventListener("touchstart", function(e){
			GAW._touchData.start = {
				y	: e.touches[0].pageY
			};
		});
		document.addEventListener("touchmove", function(e){
			GAW._touchData.end = {
				y	: e.touches[0].pageY
			};
		});
		document.addEventListener("touchend", function(e){
			var deltaY = GAW._touchData.end.y - GAW._touchData.start.y;

			// 移动超过一定距离才有效
			if (Math.abs(deltaY) > 150) {
				GAppWall.playRepeat();
			}

			e.preventDefault();
		});
	};

	/**
	 * 刷新列表，目前会追加到页尾
	 * @return {[type]} [description]
	 */
	GAppWall.playRepeat = function() {
		// @todo 最大轮播上限
		GDTMOB.reLoadAd();
	};

	/**
	 * 展示详情
	 * @return {[type]} [description]
	 */
	GAppWall.popupDetail = function(d) {
		var node, mask, link, data, addata;

		if (d.ret ===0) {
			data = d.data;
			node = document.createElement('div');
			node.className = 'wall-pop';

			if (! (mask = document.getElementById('mask'))) {
				mask = document.createElement('div');
				mask.className = 'mask';
				mask.id = 'mask';
				document.body.insertBefore(mask, document.body.firstChild);
			}

			// 补充模板填充参数
			link = 'http://android.myapp.com/android/appdetail.jsp?appid='+ d.data.appid;
			data.clickfn = 'GDTMOB.sendMessage(\'website\',{url:\''+ link +'\'});return false;';
			data.downloadfn = 'GDTMOB.click(this,\'download\', \'appwall\');return false;';
			data.hidefn = 'GAppWall.hidePopup(this);return false;';

			// 下载链接
			addata = GDTMOB.adData[GDTMOB.orderid];
			data.rl = addata.rl +'&acttype='+ GDTMOB.ACTTYPE_DOWNLOAD;
			data.cl = addata.cl;

			node.innerHTML = GDTMOB.tmpl("\n"+ document.getElementById('tplDetail').innerHTML, data);
			document.body.appendChild(node);
		} else {
			// @todo 失败重试
		}
	};

	/**
	 * 隐藏弹框
	 */
	GAppWall.hidePopup = function(el) {
		var mask,
			wrap = el.parentNode.parentNode;
		wrap.parentNode.removeChild(wrap);

		// 干掉mask
		mask = document.getElementById('mask');
		mask.parentNode.removeChild(mask);

		return false;
	};

	/**
	 * 下载
	 * @return {[type]} [description]
	 */
	GAppWall.download = function() {
		var commdata;

		commdata = GDTMOB.download(d);

		if (d) {
			// 正常，返回下载链接
			if(d.ret === 0) {
				GDTMOB.sendMessage('download', {
					url			: commdata.dstlink,
					pkgname		: commdata.pkgname,
					dlping		: commdata.dlping,	// 下载上报url
					dledping	: commdata.dledping,	// 下载完成上报url
					dlerrping	: commdata.dlerrping
				});

				// 提示用户开始下载了
				GDTMOB.downloadBtn && (GDTMOB.downloadBtn.innerHTML = '<span>下载中...</span>');
			} else if (d.data && commdata.dstlink) {
				// 如果遇到后台逻辑错误，也有可能返回正确的地址，但地址是详情页。此时调外部浏览器打开
				GDTMOB.sendMessage('website', {url: commdata.dstlink});
			} else {
				GDTMOB.downloadBtn && (GDTMOB.downloadBtn.innerHTML = '<span>重新下载</span>');
			}
		} else {
			GDTMOB.downloadBtn && (GDTMOB.downloadBtn.innerHTML = '<span>重新下载</span>');
		}
	};

	window.setInterval(function(){
		GDTMOB.sendMessage('fillparams', {'callback': 'GDTMOB.loadAd'});
	},5000);	
	
})(window.GAppWall = window.GAppWall || {});
